@@ -1,21 +1,27 @@
 /*
  * C++ interface to libheif
- * Copyright (c) 2018 struktur AG, Dirk Farin <farin@struktur.de>
  *
- * This file is part of libheif.
+ * MIT License
  *
- * libheif is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, either version 3 of
- * the License, or (at your option) any later version.
+ * Copyright (c) 2018 Dirk Farin <dirk.farin@gmail.com>
  *
- * libheif is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libheif.  If not, see <http://www.gnu.org/licenses/>.
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #ifndef LIBHEIF_HEIF_CXX_H
@@ -24,6 +30,7 @@
 #include <memory>
 #include <string>
 #include <vector>
+#include <cassert>
 
 extern "C" {
 #include <libheif/heif.h>
@@ -44,6 +51,8 @@ namespace heif {
 
     Error(const heif_error& err)
     {
+      assert(err.message);
+
       m_code = err.code;
       m_subcode = err.subcode;
       m_message = err.message;
@@ -56,7 +65,7 @@ namespace heif {
       m_message = msg;
     }
 
-    std::string get_message() const
+    const std::string& get_message() const
     { return m_message; }
 
     heif_error_code get_code() const
@@ -96,7 +105,7 @@ namespace heif {
     };
 
     // throws Error
-    void read_from_file(std::string filename, const ReadingOptions& opts = ReadingOptions());
+    void read_from_file(const std::string& filename, const ReadingOptions& opts = ReadingOptions());
 
     // DEPRECATED. Use read_from_memory_without_copy() instead.
     // throws Error
@@ -108,8 +117,7 @@ namespace heif {
     class Reader
     {
     public:
-      virtual ~Reader()
-      {}
+      virtual ~Reader() = default;
 
       virtual int64_t get_position() const = 0;
 
@@ -173,8 +181,7 @@ namespace heif {
     class Writer
     {
     public:
-      virtual ~Writer()
-      {}
+      virtual ~Writer() = default;
 
       virtual heif_error write(const void* data, size_t size) = 0;
     };
@@ -183,7 +190,7 @@ namespace heif {
     void write(Writer&);
 
     // throws Error
-    void write_to_file(std::string filename) const;
+    void write_to_file(const std::string& filename) const;
 
   private:
     std::shared_ptr<heif_context> m_context;
@@ -200,8 +207,7 @@ namespace heif {
   class ImageHandle
   {
   public:
-    ImageHandle()
-    {}
+    ImageHandle() = default;
 
     ImageHandle(heif_image_handle* handle);
 
@@ -216,6 +222,8 @@ namespace heif {
 
     bool has_alpha_channel() const noexcept;
 
+    bool is_premultiplied_alpha() const noexcept;
+
     int get_luma_bits_per_pixel() const noexcept;
 
     int get_chroma_bits_per_pixel() const noexcept;
@@ -275,7 +283,7 @@ namespace heif {
   public:
     ColorProfile_nclx();
 
-    ~ColorProfile_nclx();
+    ~ColorProfile_nclx() = default;
 
     heif_color_primaries get_color_primaries() const;
 
@@ -285,6 +293,9 @@ namespace heif {
 
     bool is_full_range() const;
 
+    void set_color_primaries(heif_color_primaries cp);
+
+    // DEPRECATED: typo in function name. Use set_color_primaries() instead.
     void set_color_primaties(heif_color_primaries cp);
 
     void set_transfer_characteristics(heif_transfer_characteristics tc);
@@ -294,10 +305,9 @@ namespace heif {
     void set_full_range_flag(bool is_full_range);
 
   private:
-    ColorProfile_nclx(heif_color_profile_nclx* nclx)
-    { mProfile = nclx; }
+    ColorProfile_nclx(heif_color_profile_nclx* nclx);
 
-    heif_color_profile_nclx* mProfile;
+    std::shared_ptr<heif_color_profile_nclx> mProfile;
 
     friend class Image;
   };
@@ -306,8 +316,7 @@ namespace heif {
   class Image
   {
   public:
-    Image()
-    {}
+    Image() = default;
 
     Image(heif_image* image);
 
@@ -335,10 +344,16 @@ namespace heif {
 
     bool has_channel(enum heif_channel channel) const noexcept;
 
+    // DEPRECATED
     const uint8_t* get_plane(enum heif_channel channel, int* out_stride) const noexcept;
 
+    // DEPRECATED
     uint8_t* get_plane(enum heif_channel channel, int* out_stride) noexcept;
 
+    const uint8_t* get_plane2(enum heif_channel channel, size_t* out_stride) const noexcept;
+
+    uint8_t* get_plane2(enum heif_channel channel, size_t* out_stride) noexcept;
+
     // throws Error
     void set_nclx_color_profile(const ColorProfile_nclx&);
 
@@ -353,6 +368,10 @@ namespace heif {
     void set_raw_color_profile(heif_color_profile_type type,
                                const std::vector<uint8_t>& data);
 
+    bool is_premultiplied_alpha() const noexcept;
+
+    void set_premultiplied_alpha(bool is_premultiplied_alpha) noexcept;
+
     class ScalingOptions
     {
     };
@@ -381,14 +400,19 @@ namespace heif {
 
     enum heif_compression_format get_compression_format() const noexcept;
 
+    // DEPRECATED: typo in function name
     bool supportes_lossy_compression() const noexcept;
 
+    // DEPRECATED: typo in function name
     bool supportes_lossless_compression() const noexcept;
 
 
     // throws Error
     Encoder get_encoder() const;
 
+    bool supports_lossy_compression() const noexcept;
+
+    bool supports_lossless_compression() const noexcept;
 
   private:
     EncoderDescriptor(const struct heif_encoder_descriptor* descr) : m_descriptor(descr)
@@ -439,21 +463,21 @@ namespace heif {
 
     std::vector<EncoderParameter> list_parameters() const noexcept;
 
-    void set_integer_parameter(std::string parameter_name, int value);
+    void set_integer_parameter(const std::string& parameter_name, int value);
 
-    int get_integer_parameter(std::string parameter_name) const;
+    int get_integer_parameter(const std::string& parameter_name) const;
 
-    void set_boolean_parameter(std::string parameter_name, bool value);
+    void set_boolean_parameter(const std::string& parameter_name, bool value);
 
-    bool get_boolean_parameter(std::string parameter_name) const;
+    bool get_boolean_parameter(const std::string& parameter_name) const;
 
-    void set_string_parameter(std::string parameter_name, std::string value);
+    void set_string_parameter(const std::string& parameter_name, const std::string& value);
 
-    std::string get_string_parameter(std::string parameter_name) const;
+    std::string get_string_parameter(const std::string& parameter_name) const;
 
-    void set_parameter(std::string parameter_name, std::string parameter_value);
+    void set_parameter(const std::string& parameter_name, const std::string& parameter_value);
 
-    std::string get_parameter(std::string parameter_name) const;
+    std::string get_parameter(const std::string& parameter_name) const;
 
   private:
     Encoder(struct heif_encoder*) noexcept;
@@ -477,7 +501,7 @@ namespace heif {
                                               [](heif_context* c) { heif_context_free(c); });
   }
 
-  inline void Context::read_from_file(std::string filename, const ReadingOptions& /*opts*/)
+  inline void Context::read_from_file(const std::string& filename, const ReadingOptions& /*opts*/)
   {
     Error err = Error(heif_context_read_from_file(m_context.get(), filename.c_str(), NULL));
     if (err) {
@@ -533,7 +557,11 @@ namespace heif {
           heif_reader_trampoline_get_position,
           heif_reader_trampoline_read,
           heif_reader_trampoline_seek,
-          heif_reader_trampoline_wait_for_file_size
+          heif_reader_trampoline_wait_for_file_size,
+          NULL,
+          NULL,
+          NULL,
+          NULL,
       };
 
   inline void Context::read_from_reader(Reader& reader, const ReadingOptions& /*opts*/)
@@ -632,7 +660,7 @@ namespace heif {
     }
   }
 
-  inline void Context::write_to_file(std::string filename) const
+  inline void Context::write_to_file(const std::string& filename) const
   {
     Error err = Error(heif_context_write_to_file(m_context.get(), filename.c_str()));
     if (err) {
@@ -669,6 +697,11 @@ namespace heif {
     return heif_image_handle_has_alpha_channel(m_image_handle.get()) != 0;
   }
 
+  inline bool ImageHandle::is_premultiplied_alpha() const noexcept
+  {
+    return heif_image_handle_is_premultiplied_alpha(m_image_handle.get()) != 0;
+  }
+
   inline int ImageHandle::get_luma_bits_per_pixel() const noexcept
   {
     return heif_image_handle_get_luma_bits_per_pixel(m_image_handle.get());
@@ -779,12 +812,15 @@ namespace heif {
 
   inline ColorProfile_nclx::ColorProfile_nclx()
   {
-    mProfile = heif_nclx_color_profile_alloc();
+    auto profile = heif_nclx_color_profile_alloc();
+    mProfile = std::shared_ptr<heif_color_profile_nclx>(profile,
+                                                        [](heif_color_profile_nclx* p) { heif_nclx_color_profile_free(p); });
   }
 
-  inline ColorProfile_nclx::~ColorProfile_nclx()
+  inline ColorProfile_nclx::ColorProfile_nclx(heif_color_profile_nclx* nclx)
   {
-    delete mProfile;
+    mProfile = std::shared_ptr<heif_color_profile_nclx>(nclx,
+                                                        [](heif_color_profile_nclx* p) { heif_nclx_color_profile_free(p); });
   }
 
   inline heif_color_primaries ColorProfile_nclx::get_color_primaries() const
@@ -799,9 +835,12 @@ namespace heif {
   inline bool ColorProfile_nclx::is_full_range() const
   { return mProfile->full_range_flag; }
 
-  inline void ColorProfile_nclx::set_color_primaties(heif_color_primaries cp)
+  inline void ColorProfile_nclx::set_color_primaries(heif_color_primaries cp)
   { mProfile->color_primaries = cp; }
 
+  inline void ColorProfile_nclx::set_color_primaties(heif_color_primaries cp)
+  { set_color_primaries(cp); }
+
   inline void ColorProfile_nclx::set_transfer_characteristics(heif_transfer_characteristics tc)
   { mProfile->transfer_characteristics = tc; }
 
@@ -889,9 +928,19 @@ namespace heif {
     return heif_image_get_plane(m_image.get(), channel, out_stride);
   }
 
+  inline const uint8_t* Image::get_plane2(enum heif_channel channel, size_t* out_stride) const noexcept
+  {
+    return heif_image_get_plane_readonly2(m_image.get(), channel, out_stride);
+  }
+
+  inline uint8_t* Image::get_plane2(enum heif_channel channel, size_t* out_stride) noexcept
+  {
+    return heif_image_get_plane2(m_image.get(), channel, out_stride);
+  }
+
   inline void Image::set_nclx_color_profile(const ColorProfile_nclx& nclx)
   {
-    Error err = Error(heif_image_set_nclx_color_profile(m_image.get(), nclx.mProfile));
+    Error err = Error(heif_image_set_nclx_color_profile(m_image.get(), nclx.mProfile.get()));
     if (err) {
       throw err;
     }
@@ -949,6 +998,15 @@ namespace heif {
     }
   }
 
+  inline bool Image::is_premultiplied_alpha() const noexcept
+  {
+    return heif_image_is_premultiplied_alpha(m_image.get()) != 0;
+  }
+
+  inline void Image::set_premultiplied_alpha(bool is_premultiplied_alpha) noexcept
+  {
+    heif_image_set_premultiplied_alpha(m_image.get(), is_premultiplied_alpha);
+  }
 
   inline Image Image::scale_image(int width, int height,
                                   const ScalingOptions&) const
@@ -981,6 +1039,7 @@ namespace heif {
                                                           maxDescriptors);
       if (nDescriptors < maxDescriptors) {
         std::vector<EncoderDescriptor> outDescriptors;
+        outDescriptors.reserve(nDescriptors);
         for (int i = 0; i < nDescriptors; i++) {
           outDescriptors.push_back(EncoderDescriptor(descriptors[i]));
         }
@@ -1014,12 +1073,22 @@ namespace heif {
 
   inline bool EncoderDescriptor::supportes_lossy_compression() const noexcept
   {
-    return heif_encoder_descriptor_supportes_lossy_compression(m_descriptor);
+    return heif_encoder_descriptor_supports_lossy_compression(m_descriptor);
+  }
+
+  inline bool EncoderDescriptor::supports_lossy_compression() const noexcept
+  {
+    return heif_encoder_descriptor_supports_lossy_compression(m_descriptor);
   }
 
   inline bool EncoderDescriptor::supportes_lossless_compression() const noexcept
   {
-    return heif_encoder_descriptor_supportes_lossless_compression(m_descriptor);
+    return heif_encoder_descriptor_supports_lossless_compression(m_descriptor);
+  }
+
+  inline bool EncoderDescriptor::supports_lossless_compression() const noexcept
+  {
+    return heif_encoder_descriptor_supports_lossless_compression(m_descriptor);
   }
 
   inline Encoder EncoderDescriptor::get_encoder() const
@@ -1143,7 +1212,7 @@ namespace heif {
     }
   }
 
-  inline void Encoder::set_integer_parameter(std::string parameter_name, int value)
+  inline void Encoder::set_integer_parameter(const std::string& parameter_name, int value)
   {
     Error err = Error(heif_encoder_set_parameter_integer(m_encoder.get(), parameter_name.c_str(), value));
     if (err) {
@@ -1151,7 +1220,7 @@ namespace heif {
     }
   }
 
-  inline int Encoder::get_integer_parameter(std::string parameter_name) const
+  inline int Encoder::get_integer_parameter(const std::string& parameter_name) const
   {
     int value;
     Error err = Error(heif_encoder_get_parameter_integer(m_encoder.get(), parameter_name.c_str(), &value));
@@ -1161,7 +1230,7 @@ namespace heif {
     return value;
   }
 
-  inline void Encoder::set_boolean_parameter(std::string parameter_name, bool value)
+  inline void Encoder::set_boolean_parameter(const std::string& parameter_name, bool value)
   {
     Error err = Error(heif_encoder_set_parameter_boolean(m_encoder.get(), parameter_name.c_str(), value));
     if (err) {
@@ -1169,7 +1238,7 @@ namespace heif {
     }
   }
 
-  inline bool Encoder::get_boolean_parameter(std::string parameter_name) const
+  inline bool Encoder::get_boolean_parameter(const std::string& parameter_name) const
   {
     int value;
     Error err = Error(heif_encoder_get_parameter_boolean(m_encoder.get(), parameter_name.c_str(), &value));
@@ -1179,7 +1248,7 @@ namespace heif {
     return value;
   }
 
-  inline void Encoder::set_string_parameter(std::string parameter_name, std::string value)
+  inline void Encoder::set_string_parameter(const std::string& parameter_name, const std::string& value)
   {
     Error err = Error(heif_encoder_set_parameter_string(m_encoder.get(), parameter_name.c_str(), value.c_str()));
     if (err) {
@@ -1187,7 +1256,7 @@ namespace heif {
     }
   }
 
-  inline std::string Encoder::get_string_parameter(std::string parameter_name) const
+  inline std::string Encoder::get_string_parameter(const std::string& parameter_name) const
   {
     const int max_size = 250;
     char value[max_size];
@@ -1199,7 +1268,7 @@ namespace heif {
     return value;
   }
 
-  inline void Encoder::set_parameter(std::string parameter_name, std::string parameter_value)
+  inline void Encoder::set_parameter(const std::string& parameter_name, const std::string& parameter_value)
   {
     Error err = Error(heif_encoder_set_parameter(m_encoder.get(), parameter_name.c_str(),
                                                  parameter_value.c_str()));
@@ -1208,7 +1277,7 @@ namespace heif {
     }
   }
 
-  inline std::string Encoder::get_parameter(std::string parameter_name) const
+  inline std::string Encoder::get_parameter(const std::string& parameter_name) const
   {
     const int max_size = 250;
     char value[max_size];
